CREATE OR REPLACE FUNCTION GET_RULE_SLOTS (I_BITS VARCHAR(3000) )
  RETURNS table(slot_id integer)
  SPECIFIC GET_RULE_SLOTS
  LANGUAGE SQL
  DETERMINISTIC
  NO EXTERNAL ACTION
  READS SQL DATA
-----------------------------------------------------------------------------
-- File:  GET_RULE_SLOTS
--
--
-- Description:  Decodes the RRF rule bit array. Assumes a varchar for bit data column is passed in. Also
--               assumes the RRF_RULE_DATA column is passed in and only the first 375 bytes (750 Hex Characters) 
--               need to be processed.
--
--
-- Called By:    webapp Analyst Workstation Grids to present the rule ids that fired on the transaction.
--
--
-- Calls:
--
--
-- Notes:
--
--
-- Usage:
--
--
-- History:    2011/11/17:  Creation date
--             2014/12/12:  Corrected Code to return the correct Slot Ids. 
--
------------------------------------------------------------------------
F1: BEGIN ATOMIC
  DECLARE slots VARCHAR(14000) DEFAULT '';
  DECLARE ii INT DEFAULT 0;
  DECLARE ch CHAR(1);
  DECLARE start INT DEFAULT 0;
  DECLARE size INT;
  DECLARE chr_no integer;
  DECLARE v_aoffset integer;
  DECLARE v_bits VARCHAR(750);
  

  set v_aoffset=ascii('A') - 10;
  --Convert to HEX and get the first 750 characters. The RRF_RULE_DATA files first 375 bytes 
  --contain the ARD and Queue Rule Firings
  set v_bits=upper(substr(hex(i_bits),1,750));
  set size = length(v_bits);
    
  while(ii < size) do
     set ch = substr(v_bits,ii+1,1);
     if(ch != '0') then 
          IF ch IN ('A','B','C','D','E','F') THEN
             set chr_no = ascii(ch) - v_aoffset;
          ELSE
             set chr_no = TO_NUMBER(ch);
          END IF;
       --  set chr_no=ascii(ch);
         set start = (ii * 4)+1;
        
        
        if(bitand(chr_no,8)=8) then
              set slots = rtrim(slots)|| rtrim(char(start)) || ',';
         end if;
         if(bitand(chr_no,4)=4) then
             set slots = rtrim(slots)|| rtrim(char(start+1)) || ',';
         end if;
         if(bitand(chr_no,2)=2) then
             set slots = rtrim(slots)|| rtrim(char(start+2)) || ',';
         end if;
         if(bitand(chr_no,1)=1) then
             set slots = rtrim(slots)|| rtrim(char(start+3)) || ',';
         end if;
        
     end if;
     set ii = ii + 1;
  end while;
  
  RETURN 
      WITH t(ordinal, index,strval) AS 
         ( VALUES ( 0, 0,'' ) 
           UNION ALL 
           SELECT ordinal+1,
                COALESCE(NULLIF(LOCATE(',', slots, index+1), 0),LENGTH(slots)+1),
                substr(slots,index+1, LOCATE(',', slots, index+1)-(index+1))
           FROM   t 
                  -- to prevent a warning condition for infinite 
                  -- recursions, we add the explicit upper 
                  -- boundary for the &quot;ordinal&quot; values 
           WHERE  ordinal < 14000 AND 
                  -- terminate if there are no further delimiters 
                  -- remaining 
                  LOCATE(',', slots, index+1) <> 0 ) 
      SELECT strval 
      FROM   t where length(trim(strval))>0;
  END
@