create or replace type sasfm_array
    as table of number
/
CREATE OR REPLACE FUNCTION GET_RULE_SLOTS (I_BITS VARchar2 )
  RETURN sasfm_array PIPELINED AS 
-----------------------------------------------------------------------------
-- File:  GET_RULE_SLOTS
--
--
-- Description:  Decodes the RRF rule bit array. Assumes a base64 encoded string is passed in. Also
--               assumes the RRF_RULE_DATA column is passed in and only the first 375 bytes (750 Hex Characters) 
--               need to be processed.
--
--
-- Called By:    webapp Analyst Workstation Grids to present the rule ids that fired on the transaction.
--
--
-- Calls:
--
--
-- Notes:
--
--
-- Usage:
--  with tx as (select smh_build_num as build_id,cmx_tran_id,rrf_rule_Data as rrf from fcm_csca where cmx_tran_id = 600000105468928 )
--    select r.rule_id,slot_id from fcm_rule r 
--    inner join fcm_build_package p on r.rule_id = p.rule_id 
--      inner join tx on tx.build_id = p.build_id  
--    inner join table(get_rule_slots(tx.rrf)) t on t.column_value=p.slot_id 
--
--
-- History:    2013/09/23:  Creation date
--             2014/11/17:  base64_decode the value
--             2014/12/12:  Corrected Code to return the correct Slot Ids. 
--
------------------------------------------------------------------------
  
  ii INTEGER := 0;
  chr_no integer :=0;
  ch char(1);
  l_start INTEGER := 0;
  l_size INTEGER;
  v_bits varchar2(750);
  v_aoffset integer:=0;
BEGIN 
	--Implicit conversion to HEX. Get the first 750 characters. The RRF_RULE_DATA files first 375 bytes 
    --contain the ARD and Queue Rule Firings
   v_bits := upper(substr(I_BITS,1,750)); 
   l_size := length(v_bits);
   v_aoffset:=ASCII('A') - 10;
    
   

  while(ii < l_size) LOOP
      ch := substr(upper(v_bits),ii+1,1);
     if(ch != '0') then 
          IF ch IN ('A','B','C','D','E','F') THEN
             chr_no := ASCII(ch) - v_aoffset;
          ELSE
             chr_no := TO_NUMBER(ch);
          END IF;
          
          l_start := (ii * 4)+1;
          
         if(bitand(chr_no,8)=8) then
             pipe row(l_start);
         end if;
         if(bitand(chr_no,4)=4) then
             pipe row(l_start+1);
         end if;
         if(bitand(chr_no,2)=2) then
             pipe row(l_start+2);
         end if;
         if(bitand(chr_no,1)=1) then
             pipe row(l_start+3);
         end if;

     end if;
      ii := ii + 1;
  end loop;

  RETURN ;

END GET_RULE_SLOTS;
/

Show errors;


